local info, addon = ...
local L = addon.L

local SET_MAIN = "main"

Library.Flux.Register( "Equipment", { subset = SET_MAIN } )

function addon.EquipSet( role, subset )
	local equipment = Inspect.Item.List( Utility.Item.Slot.Equipment() )

	local rolesub = addon.Equipment[role][subset]
	local cursub = addon.Equipment[addon.currentrole][addon.Equipment.subset]

	-- Remove all inexistant items from the item set

	for slot, item in pairs( rolesub ) do
		if item and item.id and item.id ~= cursub[slot].id then
			local itempos = Inspect.Item.Find( item.id )

			if itempos then
				itempos = Utility.Item.Slot.Parse( itempos )
			end
			if not itempos or itempos ~= "inventory" then
				local info = Inspect.Item.Detail( item.id )
				if info then
					print( L("ITEM_NOT_FOUND", info.name) )
				else
					local t, s = Utility.Item.Slot.Parse( slot )
					print( L("UNKNOWN_NOT_FOUND", s) )
				end

				rolesub[slot].id = cursub[slot].id
				rolesub[slot].slot = cursub[slot].slot
			end
		end
	end

	-- Remove old items

	for slot, id in pairs( equipment ) do
		local cur = cursub[slot]
		local new = rolesub[slot]

		if cur and new and id ~= new.id then
			if cur.slot then
				-- Just ignore unsuccessful removing, shouldn't be a big problem
				pcall( Command.Item.Move, id, cur.slot )
			end
		end
	end

	-- Equip new items

	for slot, item in pairs( rolesub ) do
		if item and item.id and item.id ~= cursub[slot].id then
			local b, err = pcall( Command.Item.Move, item.id, slot )
			if not b then
				local info = Inspect.Item.Detail( item.id )
				if info then
					print( L("EQUIP_ERROR", info.name) )
				else
					local t, s = Utility.Item.Slot.Parse( slot )
					print( L("UNKNOWN_EQUIP_ERROR", s) )
				end
			end
		end
	end
end

function addon.SaveSet( role, subsetname )
	local equipment = Inspect.Item.List( Utility.Item.Slot.Equipment() )

	local subset = addon.Equipment[role][subsetname]
	local main = addon.Equipment[role][SET_MAIN]

	for k, v in pairs( equipment ) do
		if subsetname == SET_MAIN then
			subset[k] = subset[k] or {}
			subset[k].id = v
		elseif v ~= (main[k] or {}).id then -- Save all items that differ from main set
			if rawget( subset, k ) then
				subset[k].id = v
			else
				subset[k] = { id = v }
			end
		end
	end

	addon.Equipment[role][subsetname] = subset
end

-- Save bag positions for the given subset
function addon.SaveSlots( role, subset )
	for k, v in pairs( addon.Equipment[role][subset] ) do
		if v.id then
			local slot = Inspect.Item.Find( v.id )
			if slot then
				local t = Utility.Item.Slot.Parse( slot )
				if t == "inventory" then
					v.slot = slot
				end
			end
		end
	end
end

function addon.ChangeSet( role, subset )
	local roleset = addon.Equipment[role] or {}

	-- Makes subsets fallback to main set's items
	if subset ~= SET_MAIN then
		roleset[subset] = setmetatable( roleset[subset] or {}, { __index = roleset[SET_MAIN] } )
	else
		roleset[subset] = roleset[subset] or {}
	end

	addon.Equipment[role] = roleset

	addon.SaveSet( addon.currentrole, addon.Equipment.subset )
	addon.SaveSlots( role, subset )
	addon.EquipSet( role, subset )

	addon.currentrole = role
	addon.Equipment.subset = subset
end

local function onInit()
	print( L("ADDON_LOADED", info.toc.Version) )
end

local function onUnitChange( units )
	for k, v in pairs( units ) do
		if v == "player" then
			local role = Inspect.TEMPORARY.Role()
			local subset = addon.Equipment.subset

			-- Initialize role table if it doesn't exist
			local roleset = addon.Equipment[role] or {}
			if subset ~= SET_MAIN then
				roleset[subset] = setmetatable( roleset[subset] or {}, { __index = roleset[SET_MAIN] } )
			else
				roleset[subset] = roleset[subset] or {}
			end
			addon.Equipment[role] = roleset

			addon.currentrole = role
		end
	end
end

local function onRoleChange( new )
	addon.ChangeSet( new, SET_MAIN )
end

local slash = {
	set = {
		syntax = "<name>",
		help = "Changes to the given subset.",
		func = function( line, name )
			addon.ChangeSet( addon.currentrole, name )
		end
	},
	removeset = {
		syntax = "<name>",
		help = "Removes the given subset.",
		func = function( line, name )
			if name == addon.Equipment.subset then
				addon.ChangeSet( addon.currentrole, SET_MAIN )
			end
			addon.Equipment[addon.currentrole][name] = nil

			print( L"SUBSET_REMOVED" )
		end
	},
	link = {
		syntax = "<role>",
		help = "Makes the current role share the same equipment set as the given role.",
		func = function( line, role )
			role = tonumber( role )
			if role == addon.currentrole then return end
			if not role or role < 1 or role > 6 or not addon.Equipment[role] then
				print( L"INVALID_ROLE_LINKED" )
				return
			end

			addon.Equipment[addon.currentrole] = addon.Equipment[role]
			addon.EquipSet( role, SET_MAIN )
			print( L("ROLE_LINKED", role) )
		end	
	},
	reset = {
		syntax = "",
		help = "Resets (and unlinks) current role.",
		func = function()
			addon.Equipment[addon.currentrole] = {}
			addon.Equipment.subset = SET_MAIN

			print( L"ROLE_RESET" )
		end
	},
	wipe = {
		syntax = "[<all>]",
		help = "Removes all equipment sets (all = \"true\" to wipe all profiles).",
		func = function( line, all )
			all = (all == "1" or all == "true")
			addon.Equipment:Wipe( all )
			addon.Equipment.subset = SET_MAIN
			addon.Equipment[addon.currentrole] = { main = {} }

			if all then
				print( L"EVERYTHING_WIPED" )
			else
				print( L"PROFILE_WIPED" )
			end
		end
	}
}

Library.Slash.Register( slash )

table.insert( Event.Addon.Load.End, { function( id ) if id == info.id then onInit() end end, info.id, "Init" } )

table.insert( Event.Unit.Available, { onUnitChange, info.id, "UnitChange" } )
table.insert( Event.TEMPORARY.Role, { onRoleChange, info.id, "RoleChange" } )
